class StatusCycle:
    def __init__(self, *args):
        self.list = list(args)
        self.index = 0

    @property
    def current(self):
        return self.list[self.index]

    def __str__(self):
        return self.current

    def next(self):
        self.index += 1
        try:
            return self.current
        except IndexError:
            self.index = 0
            return self.current



from asyncio import TimeoutError
from typing import Iterable, List, Union

from discord.ext.commands import Bot, Context

from discord import Client, Embed


class Paginator:
    def __init__(
        self,
        bot: Union[Client, Bot],
        pages: Iterable[Embed] = [],
        remove_reaction_after: bool = True,
        first_page_index: int = 0,
    ) -> None:
        self.bot = bot
        self.pages: List = list(pages)
        self.index = first_page_index
        self._msg = 0
        self._clear = remove_reaction_after

    def add_page(self, page: Embed) -> None:
        self.pages.append(page)

    def add_pages(self, pages: Iterable[Embed]) -> None:
        self.pages += list(pages)

    async def start(self, ctx: Context, timeout=30) -> None:
        self._msg = msg = await ctx.send(embed=self.pages[self.index])
        reaction_list = ["⏮", "◀", "⏹", "▶", "⏭"]
        for reaction in reaction_list:
            await msg.add_reaction(reaction)
        try:
            while True:
                reaction, user = await self.bot.wait_for(
                    "reaction_add",
                    check=lambda r, u: not u.bot
                    and str(r.emoji) in reaction_list
                    and r.message == msg,
                    timeout=timeout,
                )
                emoji = str(reaction.emoji)
                edit = True
                if emoji == "▶":
                    if self.index + 1 == len(self.pages):
                        edit = False
                    else:
                        self.index += 1
                elif emoji == "◀":
                    if self.index == 0:
                        edit = False
                    else:
                        self.index -= 1
                elif emoji == "⏹":
                    await msg.clear_reactions()
                    return
                elif emoji == "⏭":
                    self.index = len(self.pages) - 1
                elif emoji == "⏮":
                    self.index = 0
                if self._clear:
                    await msg.remove_reaction(emoji, user)
                if edit:
                    await msg.edit(embed=self.pages[self.index])
        except TimeoutError:
            await msg.clear_reactions()

    async def stop(self) -> None:
        if self._msg != 0:
            await self._msg.clear_reactions()



import discord


class Webhooks:
    @staticmethod
    async def find_and_send(
        text: str,
        *,
        channel: discord.TextChannel,
        webhook_name: str,
        sender_name: str = None,
        icon_url: str = None
    ) -> discord.WebhookMessage:
        webhooks = await channel.webhooks()
        for webhook in webhooks:
            if webhook.name == webhook_name:
                if not icon_url:
                    icon_url = webhook.avatar
                if not sender_name:
                    sender_name = webhook.name
                return await webhook.send(
                    text, username=sender_name, avatar_url=icon_url
                )
        else:
            raise RuntimeError("No webhook was found named {}".format(webhook_name))

    @staticmethod
    async def send_message(
        text: str,
        webhook: discord.Webhook,
        *,
        sender_name: str = None,
        icon_url: str = None
    ) -> discord.WebhookMessage:
        if not icon_url:
            icon_url = webhook.avatar
        if not sender_name:
            sender_name = webhook.name
        return await webhook.send(text, username=sender_name, avatar_url=icon_url)



import aiohttp


async def chatbot(message: str, *, api_key: str, language: str = "en") -> str:
    headers = {"x-api-key": api_key}
    params = {"type": "stable", "message": message}
    async with aiohttp.ClientSession() as ses:
        async with ses.get(
            "https://api.pgamerx.com/v3/ai/response", headers=headers, params=params
        ) as response:
            return await response.json()



import asyncio
from inspect import iscoroutinefunction
from typing import Coroutine, Union

import aiosqlite

import discord
from discord.ext import commands


class Database:
    def __init__(self, bot: Union[commands.Bot, discord.Client], filename: str):
        self.filename = filename
        self.bot = bot
        self.invite_tracker = self.InviteTracker(bot, filename)

    def custom_prefix(self, default: str) -> Coroutine:
        async def get_prefix(_, message):
            try:
                async with aiosqlite.connect(self.filename) as con:
                    async with con.cursor() as cur:
                        await cur.execute(
                            "CREATE TABLE IF NOT EXISTS prefixes (guild_id BIGINT PRIMARY KEY, prefix TEXT)"
                        )
                        await cur.execute(
                            "SELECT prefix FROM prefixes WHERE guild_id=?",
                            (message.guild.id,),
                        )
                        prefix = await cur.fetchall()
                        if prefix == []:
                            await cur.execute(
                                f"INSERT INTO prefixes VALUES (?, ?)",
                                (message.guild.id, default),
                            )
                            await con.commit()
                            prefix = [default]
                        return prefix[0]
            except AttributeError:
                return default

        return get_prefix

    async def reply_with_prefix(self, message: discord.Message) -> discord.Message:
        prefix = self.bot.command_prefix
        if callable(prefix):
            if iscoroutinefunction(prefix):
                prefix = await prefix(self.bot, message)
            else:
                prefix = prefix(self.bot, message)
        return await message.channel.send(f"My prefix is `{prefix}`")

    async def change_prefix(self, guild_id: int, prefix: str):
        async with aiosqlite.connect(self.filename) as con:
            async with con.cursor() as cur:
                await cur.execute(
                    f"UPDATE prefixes SET prefix=? WHERE guild_id=?", (prefix, guild_id)
                )
                await con.commit()

    class InviteTracker:
        def __init__(self, bot, filename):
            self.bot = bot
            self.filename = filename
            asyncio.get_event_loop().create_task(self._init())

        async def _init(self):
            self.data = {
                g.id: {i.id: i.uses for i in await g.invites()} for g in self.bot.guilds
            }
            async with aiosqlite.connect(self.filename) as con:
                async with con.cursor() as cur:
                    await cur.execute(
                        "CREATE TABLE IF NOT EXISTS invites (guild_id BIGINT, invite_id TEXT, uses INT, PRIMARY KEY (guild_id, invite_id))"
                    )
                    await con.commit()
                    await cur.execute("SELECT * FROM invites")
                    data = await cur.fetchall()
                    if data == []:
                        await self._add_to_db(self.data, True)

        async def _add_to_db(self, data, init=False, *, guild_id=0):
            async with aiosqlite.connect(self.filename) as con:
                async with con.cursor() as cur:
                    if init:
                        for guild_id in data.keys():
                            for invite_id in data[guild_id].keys():
                                await cur.execute(
                                    "INSERT INTO invites VALUES (?, ?, ?)",
                                    (guild_id, invite_id, data[guild_id][invite_id]),
                                )
                    else:
                        for invite_id in data.keys():
                            await cur.execute(
                                "INSERT INTO invites VALUES (?, ?, ?)",
                                (guild_id, invite_id, data[invite_id]),
                            )
                    await con.commit()

        async def _remove_from_db(self, what, data):
            async with aiosqlite.connect(self.filename) as con:
                async with con.cursor() as cur:
                    await cur.execute(
                        f"DELETE FROM invites WHERE {'invite_id' if what == 'invite' else 'guild_id'} = ?",
                        (data,),
                    )
                    await con.commit()

        async def add_guild(self, guild):
            self.data[guild.id] = {i.id: i.uses for i in await guild.invites()}
            await self._add_to_db(self.data[guild.id], guild.id)

        async def remove_guild(self, guild):
            del self.data[guild.id]
            await self._remove_from_db("guild", guild.id)

        async def add_invite(self, invite):
            self.data[invite.guild.id][invite.id] = invite.uses
            await self._add_to_db(
                self.data[invite.guild.id][invite.id], invite.guild.id
            )

        async def remove_invite(self, invite):
            del self.data[invite.guild.id][invite.id]
            await self._remove_from_db("invite", invite.id)

        async def increment_uses(self, invite, count):
            self.data[invite.guild.id][invite.id] += count
            async with aiosqlite.connect(self.filename) as con:
                async with con.cursor() as cur:
                    await cur.execute(
                        "UPDATE invites SET uses = ? WHERE invite_id = ?",
                        (self.data[invite.guild.id][invite.id], invite.id),
                    )
                    await con.commit()

        async def track(self, member):
            new_data = {i.id: i.uses for i in await member.guild.invites()}
            for inv in self.data[member.guild.id].keys():
                if new_data[inv] == self.data[member.guild.id][inv] + 1:
                    for invite in await member.guild.invites():
                        if invite.id == inv:
                            return [member.guild.get_member(invite.inviter.id), invite]



"""
A helper module for discord.py
Made by Dorukyum on GitHub
"""

from .chatbot import *
from .database import *
from .embed import *
from .paginator import *
from .requests import *
from .status import *
from .webhooks import *

__author__ = "Dorukyum"
__license__ = "MIT"
__version__ = "0.0.5"



from typing import Union

from discord import Color, Colour
from discord import Embed as OriginalEmbed


def Embed(
    title: str = "",
    description: str = "",
    colour: Union[int, Colour, Color] = 0x202225,
    fields: dict = {},
    inline_fields: bool = True,
    footer: str = "",
    footer_icon: str = None,
    author: str = "",
    author_icon: str = None,
    thumbnail: str = None,
    image: str = None,
):
    embed = OriginalEmbed(title=title, description=description, colour=colour)
    if footer_icon is not None:
        embed.set_footer(text=footer, icon_url=footer_icon)
    else:
        embed.set_footer(text=footer)
    if author_icon is not None:
        embed.set_author(name=author, icon_url=author_icon)
    else:
        embed.set_author(name=author)
    if thumbnail is not None:
        embed.set_thumbnail(url=thumbnail)
    if image is not None:
        embed.set_image(url=image)
    for name, value in fields.items():
        embed.add_field(name=name, value=value, inline=inline_fields)
    return embed



import aiohttp


class Requests:
    @staticmethod
    async def get(url, **kwargs):
        async with aiohttp.ClientSession() as ses:
            async with ses.get(url, **kwargs) as resp:
                return resp

    @staticmethod
    async def post(url, **kwargs):
        async with aiohttp.ClientSession() as ses:
            async with ses.post(url, **kwargs) as resp:
                return resp



from setuptools import setup

classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "Topic :: Software Development :: Build Tools",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3",
]

setup(
    name="discord-helpers",
    packages=["discord.ext.helpers"],
    version="0.0.5",
    license="MIT",
    description="A helper module for discord.py",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Dorukyum",
    url="https://github.com/Dorukyum/discord-helpers",
    keywords="discord, discord.py, API",
    install_requires=open("requirements.txt").read().split("\n"),
    classifiers=classifiers,
    project_urls={"Source": "https://github.com/Dorukyum/discord-helpers"},
)
